/*
 *  Copyright (C) 2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#ifndef __KIBA_AKAMARU_H__
#define __KIBA_AKAMARU_H__

#include <glib.h>
#include <gdk/gdk.h>
#include <akamaru.h>

G_BEGIN_DECLS

typedef struct _KibaAkamaruObject  KibaAkamaruObject;
typedef struct _KibaAkamaruModel   KibaAkamaruModel;

typedef void (*KibaAkamaruFunc) (KibaAkamaruModel*, KibaAkamaruObject*, gpointer);

enum {
	AKAMARU_DOCK = 0,
	AKAMARU_ROPE,
	AKAMARU_WOBBLY,
	AKAMARU_SPRING,
};

struct _KibaAkamaruModel {
	GdkRectangle geometry;
	gboolean  active;
	gboolean  rope_spring_while_dragging;
	float    rope_string_length;
	float    throw_gravity;
	float    gravity;
	float    inertia;
	float    wobbly_spring;
	
	int       constrain_iterations;
	float    elasticity;
	int       friction;
	float    k;
	
	AkamaruModel        model;
	int                 model_flag;          /* model flag */
	AkamaruObject      *anchor_object;
	AkamaruAnchor      *anchor;              /* anchor for object gravity */
	AkamaruAnchor      *mouse_anchor;        /* anchor for object dragging */
	AkamaruPolygon     *desktop_polygon;     /* polygon around the desktop (for object reflection) */
};

struct _KibaAkamaruObject {
	KibaAkamaruModel *model;
	AkamaruObject    *object;           /* Physical Object */
	AkamaruSpring    *spring;           /* Spring from KibaAkamaruObject to the Dock anchor */
	AkamaruString    *rope_string;      /* String from KibaAkamaruObject to the next one */
	GList            *wobbly_spring;    /* Spring to all other KibaAkamaruObjects */
};

void kiba_akamaru_set_model_from_string (KibaAkamaruModel *model,
					        const char       *string);

void kiba_akamaru_set_k (KibaAkamaruModel *model,
			 float            k);

void kiba_akamaru_set_friction (KibaAkamaruModel *model,
				float            friction);

void kiba_akamaru_set_elasticity (KibaAkamaruModel *model,
				  float            elasticity);

void kiba_akamaru_set_wobbly_spring (KibaAkamaruModel *model,
				     float            wobbly_spring);

void kiba_akamaru_set_inertia (KibaAkamaruModel *model,
			       float            inertia);

void kiba_akamaru_set_gravity (KibaAkamaruModel *model,
			       float            gravity);

void kiba_akamaru_set_throw_gravity (KibaAkamaruModel *model,
				     float            throw_gravity);

void kiba_akamaru_set_rope_string_length (KibaAkamaruModel *model,
					  float            rope_string_length);

void kiba_akamaru_model_enable (KibaAkamaruModel *model,
				gboolean          enable);

gboolean kiba_akamaru_is_enabled (KibaAkamaruModel *model);

void kiba_akamaru_unset_mouse_anchor (KibaAkamaruModel *model);

void kiba_akamaru_set_mouse_anchor (KibaAkamaruModel  *model,
				    KibaAkamaruObject *object,
				    float             x,
				    float             y);

void kiba_akamaru_set_anchor_position (KibaAkamaruModel *model,
				       int               x,
				       int               y);

void kiba_akamaru_update_mouse_anchor (KibaAkamaruModel *model,
				       int               new_x,
				       int               new_y,
				       float            icon_size,
				       GdkRectangle      screen_geometry);

void kiba_akamaru_get_mouse_anchor_position (KibaAkamaruModel *model,
					     int              *anchor_x,
					     int              *anchor_y);

void kiba_akamaru_model_set_size (KibaAkamaruModel *model,
				  GdkRectangle      screen_geometry);

void kiba_akamaru_model_step (KibaAkamaruModel *model);

void kiba_akamaru_model_free (KibaAkamaruModel *model);

KibaAkamaruModel* kiba_akamaru_model_new (GdkRectangle screen_geometry);

void kiba_akamaru_reload_spacers (KibaAkamaruModel *model);

void kiba_akamaru_object_throw (KibaAkamaruModel   *model,
				KibaAkamaruObject *object,
				int                dx,
				int                dy);

void kiba_akamaru_object_bounce (KibaAkamaruModel   *model,
				 KibaAkamaruObject *object,
				 GdkRectangle       bounce_point);

void kiba_akamaru_object_set_gravity_point (KibaAkamaruModel   *model,
					    KibaAkamaruObject *object,
					    GdkRectangle       gravity_normal);

void kiba_akamaru_object_set_position (KibaAkamaruObject *object,
				       float             x,
				       float             y);

void kiba_akamaru_object_free (KibaAkamaruModel   *model,
			       KibaAkamaruObject *object);

KibaAkamaruObject* kiba_akamaru_add_object (KibaAkamaruModel *model,
					    gint              width,
					    gint              height,
					    float            radius);

void kiba_akamaru_do_for_all_objects (KibaAkamaruModel *model,
				      KibaAkamaruFunc   func,
				      gpointer          data);

G_END_DECLS

#endif
