#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include <math.h>

#include "mixer.h"
#include "gst_mixer.h"

#define GSTMIXER(x) ((gst_mixer_t *)x->priv)
static mixer_ops_t *get_mixer_ops(void);

static void
error(const char *fmt, ...) {
  va_list va;

  va_start(va, fmt);
  fprintf(stderr, "kiba-volume gst_mixer: ");
  vfprintf(stderr, fmt, va);
  fprintf(stderr, "\n");
  va_end(va);
}


static gboolean
mixer_filter_func (GstMixer * mixer, gpointer user_data)
{
  GstElementFactory *factory;
  const gchar *long_name;
  gchar *devname = NULL;
  gchar *name;
  gint *p_count = (gint *) user_data;

  /* fetch name */
  if (g_object_class_find_property (G_OBJECT_GET_CLASS (G_OBJECT (mixer)),
                                    "device-name")) {
    g_object_get (mixer, "device-name", &devname, NULL);
    GST_DEBUG ("device name: %s", GST_STR_NULL (devname));
  } else {
    devname = NULL;
    GST_DEBUG ("device name unknown, no 'device-name' property");
  }
    
  factory = gst_element_get_factory (GST_ELEMENT (mixer));
  long_name = gst_element_factory_get_longname (factory);

  if (devname) {
    name = g_strdup_printf ("%s (%s)", devname, long_name);
    g_free (devname);
  } else {
    gchar *title;

    *p_count += 1;

    title = g_strdup_printf ("Unknown Volume Control %d",  *p_count);
    name = g_strdup_printf ("%s (%s)", title, long_name);
    g_free (title);
  }
  
  g_object_set_data_full (G_OBJECT (mixer),
                          "kiba-dock",
                          name,
                          (GDestroyNotify) g_free);

  GST_DEBUG ("Adding '%s' to list of available mixers", name);

  return TRUE; /* add mixer to list */
}

static GList *
create_mixer_collection (void)
{
  GList *mixer_list;
  gint counter = 0;

  mixer_list = gst_audio_default_registry_mixer_filter (mixer_filter_func,
                                                        FALSE,
                                                        &counter);
  return mixer_list;
}

static GstMixerTrack *
get_master_track(mixer_t *mixer){
  const GList *track_list = gst_mixer_list_tracks(GSTMIXER(mixer)->mixer);
  const GList *tracks;
  for ( tracks = track_list;tracks!=NULL;tracks=tracks->next){
	if ( GST_MIXER_TRACK_HAS_FLAG((GstMixerTrack*)tracks->data,GST_MIXER_TRACK_MASTER) ){
		return (GstMixerTrack*)tracks->data;
	}
  } 

}

static mixer_t *
gst_mixer_open(char *card) {
  mixer_t *result;
  gst_mixer_t *gst_result;
  gchar *devname = NULL;

  result = (mixer_t *) malloc(sizeof(mixer_t));
  gst_result = (gst_mixer_t *) malloc(sizeof(gst_mixer_t));
  result->priv = (void*)gst_result;
  result->ops = get_mixer_ops();
  
  gst_result->mixer_list = create_mixer_collection();
  GList *mixer;
  
  for( mixer = gst_result->mixer_list ; mixer != NULL; mixer  = mixer->next ){
  	g_object_get (mixer->data, "device-name", &devname, NULL);
  	if ( g_strrstr( devname,card ) == 0 ){
		gst_result->mixer = (GstMixer*)mixer->data;
		break;
 	}
  }
   
  result->name = g_strdup_printf("%s", devname);
  result->nrdevices = g_list_length(mixer);

  return result;
}

static void
gst_mixer_close(mixer_t * mixer) {

 GList* gst_mixer;

 for( gst_mixer = GSTMIXER(mixer)->mixer_list ; gst_mixer != NULL; gst_mixer  = gst_mixer->next ){
          gst_element_set_state (GST_ELEMENT (gst_mixer->data), GST_STATE_NULL);
	  gst_object_unref(gst_mixer->data);
  }
 
  g_list_free(GSTMIXER(mixer)->mixer_list);
  free(mixer->priv);
  free(mixer);
}

/* get the full scale of a device and get/set the volume */
static long
gst_mixer_device_get_fullscale(mixer_t * mixer, int devid) {
  return get_master_track(mixer)->max_volume;
}

/* Fuction to convert from volume to percentage. val = volume */
static int
convert_prange(int val, int min, int max) {
  int range = max - min;
  int tmp;

  if (range == 0)
    return 0;
  val -= min;
  tmp = rint((double) val / (double) range * 100);
  return tmp;
}

/* Function to convert from percentage to volume. val = percentage */
static int
convert_prange1(int val, int min, int max) {
  int range = max - min;
  int tmp;

  if (range == 0)
    return 0;

  tmp = rint((double) range * ((double) val * .01)) + min;
  return tmp;
}

void
gst_mixer_device_get_volume(mixer_t * mixer, int devid, 
                             int *left, int *right) {

  long min, max, lvol, rvol;
  gint *volumes;
  GstMixerTrack* track = get_master_track(mixer);
  
  max = track->max_volume;
  min = track->min_volume;

  volumes = g_new (gint, track->num_channels);
  gst_mixer_get_volume (GSTMIXER(mixer)->mixer, track, volumes);
  if ( track->num_channels > 0 )
	lvol = volumes[0];
  if ( track->num_channels > 1 )
	rvol = volumes[1];

  g_free (volumes);
  
  *left = convert_prange(lvol, min, max);
  *right = convert_prange(rvol, min, max);
}

static void
gst_mixer_device_set_volume(mixer_t * mixer, int devid, int left, int right) {
 
  GstMixerTrack* track = get_master_track(mixer);
  gint *volumes;
  long min, max, lvol, rvol;

  volumes = g_new (gint, track->num_channels);
 
  max = track->max_volume;
  min = track->min_volume;
  
  lvol = convert_prange1(left, min, max);
  rvol = convert_prange1(right, min, max);

  if ( track->num_channels > 0 )
	volumes[0] = lvol;
  if ( track->num_channels > 1 )
	volumes[1] = rvol;

  gst_mixer_set_volume (GSTMIXER(mixer)->mixer, track, volumes);

  g_free (volumes);

}

mixer_idz_t *
gst_mixer_get_id_list(void) {
  mixer_idz_t *result = NULL;
 GList *mixer_list;
 GList *mixer;
 gchar *devname = NULL;
  char name[64];

  mixer_list = create_mixer_collection();

  for( mixer = mixer_list; mixer != NULL; mixer  = mixer->next ){
	  g_object_get (mixer->data, "device-name", &devname, NULL);
          sprintf(name, "%s", devname);
          g_free (devname);
          result = mixer_id_list_add(name, result);
          gst_element_set_state (GST_ELEMENT (mixer->data), GST_STATE_NULL);
	  gst_object_unref(mixer->data);
  }
 
  g_list_free(mixer_list);
  return result;
}

static mixer_ops_t gst_mixer_ops = {
  .mixer_get_id_list = gst_mixer_get_id_list,
  .mixer_open = gst_mixer_open,
  .mixer_close = gst_mixer_close,
  .mixer_device_get_fullscale = gst_mixer_device_get_fullscale,
  .mixer_device_get_volume = gst_mixer_device_get_volume,
  .mixer_device_set_volume = gst_mixer_device_set_volume
};

static mixer_ops_t *
get_mixer_ops(void) {
  return &gst_mixer_ops;
}

mixer_ops_t *
init_gst_mixer(void) {
 
  gst_init(NULL,NULL);
  return get_mixer_ops();
}
